@@ -28,12 +28,19 @@
 #include <glom/libglom/data_structure/glomconversions.h>
 #include <glom/libglom/data_structure/layout/report_parts/layoutitem_summary.h>
 #include <glom/libglom/data_structure/layout/report_parts/layoutitem_fieldsummary.h>
+#include <glom/reports/report_builder.h>
+#include <glom/mode_design/dialog_add_related_table.h>
+#include <glom/mode_design/script_library/dialog_script_library.h>
 #include "relationships_overview/dialog_relationships_overview.h"
 #include "filechooser_export.h"
+#include <glom/glom_privs.h>
 #include <sstream> //For stringstream.
 #include <fstream>
 #include <glibmm/i18n.h>
 
+namespace Glom
+{
+
 Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
 : PlaceHolder(cobject, refGlade),
   m_pLabel_Name(0),
@@ -54,6 +61,7 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
   m_pDialog_Reports(0),
   m_pDialog_Fields(0),
   m_pDialog_Relationships(0),
+  m_dialog_addrelatedtable(0),
   m_pDialogConnection(0),
   m_pDialogConnectionFailed(0),
   m_pDialogLayoutReport(0)
@@ -145,18 +153,6 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
     std::cerr << ex.what() << std::endl;
   }
 
-  try
-  {
-    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_design");
-
-    refXml->get_widget_derived("window_design", m_pDialog_Fields);
-    m_pDialog_Fields->signal_hide().connect( sigc::mem_fun(*this, &Frame_Glom::on_developer_dialog_hide));
-  }
-  catch(const Gnome::Glade::XmlError& ex)
-  {
-    std::cerr << ex.what() << std::endl;
-  }
-
   try
   {
     Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_design");
@@ -213,7 +209,6 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
   //This means that set_document and load/save are delegated to these children:
   add_view(m_pBox_Tables);
   add_view(m_pBox_Reports);
-  add_view(m_pDialog_Fields); //Also a composite view.
   add_view(m_pDialog_Relationships); //Also a composite view.
   add_view(m_pDialogConnection); //Also a composite view.
   add_view(&m_Notebook_Data); //Also a composite view.
@@ -274,6 +269,13 @@ Frame_Glom::~Frame_Glom()
     delete m_pDialog_Fields;
     m_pDialog_Fields = 0;
   }
+
+  if(m_dialog_addrelatedtable)
+  {
+    remove_view(m_dialog_addrelatedtable);
+    delete m_dialog_addrelatedtable;
+    m_dialog_addrelatedtable = 0;
+  }
 }
 
 void Frame_Glom::set_databases_selected(const Glib::ustring& strName)
@@ -469,7 +471,7 @@ void Frame_Glom::on_menu_userlevel_Developer(const Glib::RefPtr<Gtk::RadioAction
 
       //Check whether the current user has developer privileges:
       const ConnectionPool* connection_pool = ConnectionPool::get_instance();
-      bool test = get_user_is_in_group(connection_pool->get_user(), GLOM_STANDARD_GROUP_NAME_DEVELOPER);
+      bool test = Privs::get_user_is_in_group(connection_pool->get_user(), GLOM_STANDARD_GROUP_NAME_DEVELOPER);
       if(test)
       {
         std::cout << "DEBUG: User=" << connection_pool->get_user() << " _is_ in the developer group on the server." << std::endl;
@@ -491,6 +493,21 @@ void Frame_Glom::on_menu_userlevel_Developer(const Glib::RefPtr<Gtk::RadioAction
         dialog.set_secondary_text(_("Developer mode is not available. Check that you have sufficient database access rights and that the glom file is not read-only."));
         dialog.set_transient_for(*get_app_window());
         dialog.run();
+      }
+      else if(document->get_document_format_version() < Document_Glom::get_latest_known_document_format_version())
+      {
+        Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Saving in New Document Format")), true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE);
+        dialog.set_secondary_text(_("The document was created by an earlier version of the application. Making changes to the document will mean that the document cannot be opened by some earlier versions of the application."));
+        dialog.set_transient_for(*get_app_window());
+        dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+        dialog.add_button(_("Continue"), Gtk::RESPONSE_OK);
+        const int response = dialog.run();
+        test = (response == Gtk::RESPONSE_OK);
+      }
+
+      if(!test)
+      {
+        //Abort the change of user level:
 
         //This causes an endless loop, but it is not recursive so we can't block it.
         //TODO: Submit GTK+ bug.
@@ -526,7 +543,7 @@ void Frame_Glom::on_menu_file_export()
   g_assert(pWindowApp);
 
   //Do not try to export the data if the user may not view it:
-  Privileges table_privs = get_current_privs(m_table_name);
+  Privileges table_privs = Privs::get_current_privs(m_table_name);
   if(!table_privs.m_view)
   {
     show_ok_dialog(_("Export Not Allowed."), _("You do not have permission to view the data in this table, so you may not export the data."), *pWindowApp, Gtk::MESSAGE_ERROR);
@@ -569,7 +586,7 @@ void Frame_Glom::export_data_to_string(Glib::ustring& the_string, const FoundSet
   if(fieldsSequence.empty())
     return;
 
-  const Glib::ustring query = GlomUtils::build_sql_select_with_where_clause(found_set.m_table_name, fieldsSequence, found_set.m_where_clause, found_set.m_sort_clause);
+  const Glib::ustring query = Utils::build_sql_select_with_where_clause(found_set.m_table_name, fieldsSequence, found_set.m_where_clause, found_set.m_sort_clause);
 
   //TODO: Lock the database (prevent changes) during export.
   Glib::RefPtr<Gnome::Gda::DataModel> result = query_execute(query, get_app_window());
@@ -618,7 +635,7 @@ void Frame_Glom::export_data_to_stream(std::ostream& the_stream, const FoundSet&
   if(fieldsSequence.empty())
     return;
 
-  const Glib::ustring query = GlomUtils::build_sql_select_with_where_clause(found_set.m_table_name, fieldsSequence, found_set.m_where_clause, found_set.m_sort_clause);
+  const Glib::ustring query = Utils::build_sql_select_with_where_clause(found_set.m_table_name, fieldsSequence, found_set.m_where_clause, found_set.m_sort_clause);
 
   //TODO: Lock the database (prevent changes) during export.
   Glib::RefPtr<Gnome::Gda::DataModel> result = query_execute(query, get_app_window());
@@ -650,7 +667,7 @@ void Frame_Glom::export_data_to_stream(std::ostream& the_stream, const FoundSet&
 
             if(layout_item->get_glom_type() == Field::TYPE_IMAGE) //This is too much data.
             {
-              if(!GlomConversions::value_is_empty(value))
+              if(!Conversions::value_is_empty(value))
                 std::cout << "  field name=" << layout_item->get_name() << ", image value not empty=" << std::endl;
             }
             //std::cout << "  field name=" << layout_item->get_name() << ", value=" << layout_item->m_field.sql(value) << std::endl;
@@ -740,6 +757,139 @@ void Frame_Glom::on_menu_Tables_EditTables()
   do_menu_Navigate_Table(); 
 }
 
+void Frame_Glom::on_menu_Tables_AddRelatedTable()
+{
+  //Delete and recreate the dialog,
+  //so we start with a blank one:
+  if(m_dialog_addrelatedtable)
+  {
+    remove_view(m_dialog_addrelatedtable);
+    delete m_dialog_addrelatedtable;
+    m_dialog_addrelatedtable = 0;
+  }
+
+  try
+  {
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_add_related_table");
+
+    refXml->get_widget_derived("dialog_add_related_table", m_dialog_addrelatedtable);
+  }
+  catch(const Gnome::Glade::XmlError& ex)
+  {
+    std::cerr << ex.what() << std::endl;
+  }
+
+  if(!m_dialog_addrelatedtable)
+    return;
+
+  add_view(m_dialog_addrelatedtable); //Give it access to the document.
+  m_dialog_addrelatedtable->set_fields(m_table_name);
+
+  m_dialog_addrelatedtable->signal_request_edit_fields().connect( sigc::mem_fun(*this, &Frame_Glom::on_dialog_add_related_table_request_edit_fields) );
+
+  m_dialog_addrelatedtable->signal_response().connect( sigc::mem_fun(*this, &Frame_Glom::on_dialog_add_related_table_response) );
+
+  Gtk::Window* parent = get_app_window();
+
+  if(parent)
+    m_dialog_addrelatedtable->set_transient_for(*parent);
+
+  m_dialog_addrelatedtable->set_modal(); //We don't want people to edit the main window while we are changing structure.
+  m_dialog_addrelatedtable->show();
+}
+
+void Frame_Glom::on_dialog_add_related_table_response(int response)
+{
+  if(!m_dialog_addrelatedtable)
+    return;
+
+  m_dialog_addrelatedtable->hide();
+
+  bool stop_trying = false;
+  if(response == Gtk::RESPONSE_OK)
+  {
+    Glib::ustring table_name, relationship_name, from_key_name;
+    m_dialog_addrelatedtable->get_input(table_name, relationship_name, from_key_name);
+
+    Gtk::Window* parent = get_app_window();
+
+    //It would be nice to put this in the dialog's on_response() instead,
+    //but I don't think we can stop the response from being returned. murrayc
+    if(get_table_exists_in_database(table_name))
+    {
+      Frame_Glom::show_ok_dialog(_("Table Exists Already"), _("A table with this name already exists in the database. Please choose a different table name."), *parent, Gtk::MESSAGE_ERROR);
+    }
+    else if(get_relationship_exists(m_table_name, relationship_name))
+    {
+      Frame_Glom::show_ok_dialog(_("Relationship Exists Already"), _("A relationship with this name already exists for this table. Please choose a different relationship name."), *parent, Gtk::MESSAGE_ERROR);
+    }
+    else if(table_name.empty() || relationship_name.empty() || relationship_name.empty())
+    {
+      Frame_Glom::show_ok_dialog(_("More information needed"), _("You must specify a field, a table name, and a relationship name."), *parent, Gtk::MESSAGE_ERROR);
+    }
+    else
+    {
+      stop_trying = true;
+    }
+
+    if(!stop_trying)
+    {
+      //Offer the dialog again:
+      //This signal handler should be called again when a button is clicked.
+      m_dialog_addrelatedtable->show();
+    }
+    else
+    {
+      //Create the new table:
+      const bool result = create_table_with_default_fields(table_name);
+      if(!result)
+      {
+        std::cerr << "Frame_Glom::on_menu_Tables_AddRelatedTable(): create_table_with_default_fields() failed." << std::endl;
+        return;
+      }
+   
+      //Create the new relationship: 
+      sharedptr<Relationship> relationship = sharedptr<Relationship>::create();
+
+      relationship->set_name(relationship_name);
+      relationship->set_title(Utils::title_from_string(relationship_name));
+      relationship->set_from_table(m_table_name);
+      relationship->set_from_field(from_key_name);
+      relationship->set_to_table(table_name);
+
+      sharedptr<Field> related_primary_key = get_field_primary_key_for_table(table_name); //This field was created by create_table_with_default_fields().
+      if(!related_primary_key)
+      {
+        std::cerr << "Frame_Glom::on_menu_Tables_AddRelatedTable(): get_field_primary_key_for_table() failed." << std::endl;
+        return;
+      }
+
+      relationship->set_to_field(related_primary_key->get_name());
+
+      relationship->set_allow_edit(true);
+      relationship->set_auto_create(true);
+
+      Document_Glom* document = get_document();
+      if(!document)
+        return;
+
+      document->set_relationship(m_table_name, relationship);
+
+      on_dialog_tables_hide(); //Update the menu.
+
+      Gtk::Window* parent = get_app_window();
+      if(parent)
+        show_ok_dialog(_("Related Table Created"), _("The new related table has been created."), *parent, Gtk::MESSAGE_INFO);
+    }
+  }
+}
+
+void Frame_Glom::on_dialog_add_related_table_request_edit_fields()
+{
+  if(m_dialog_addrelatedtable)
+    do_menu_developer_fields(*m_dialog_addrelatedtable);
+}
+
 void Frame_Glom::do_menu_Navigate_Table(bool open_default)
 {
   if(get_document()->get_connection_database().empty())
@@ -1041,7 +1191,7 @@ void Frame_Glom::on_menu_developer_database_preferences()
       add_view(dialog);
       dialog->load_from_document();
 
-      dialog->run();
+     Glom::Utils::dialog_run_with_help(dialog, "dialog_database_preferences");
 
       remove_view(dialog);
       delete dialog;
@@ -1057,6 +1207,14 @@ void Frame_Glom::on_menu_developer_database_preferences()
 }
 
 void Frame_Glom::on_menu_developer_fields()
+{
+  Gtk::Window* parent = get_app_window();
+  if(parent)
+    do_menu_developer_fields(*parent);
+
+}
+
+void Frame_Glom::do_menu_developer_fields(Gtk::Window& parent)
 {
   //Check that there is a table to show:
   if(m_table_name.empty())
@@ -1065,7 +1223,24 @@ void Frame_Glom::on_menu_developer_fields()
   }
   else
   {
-    m_pDialog_Fields->set_transient_for(*get_app_window());
+    if(!m_pDialog_Fields)
+    {
+       try
+       {
+         Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_design");
+
+         refXml->get_widget_derived("window_design", m_pDialog_Fields);
+         m_pDialog_Fields->signal_hide().connect( sigc::mem_fun(*this, &Frame_Glom::on_developer_dialog_hide));
+       }
+       catch(const Gnome::Glade::XmlError& ex)
+       {
+         std::cerr << ex.what() << std::endl;
+       }
+
+       add_view(m_pDialog_Fields);
+    }
+
+    m_pDialog_Fields->set_transient_for(parent);
     m_pDialog_Fields->init_db_details(m_table_name);
     m_pDialog_Fields->show();
   }
@@ -1084,7 +1259,7 @@ void Frame_Glom::on_menu_developer_relationships_overview()
       add_view(dialog);
       dialog->load_from_document();
 
-      dialog->run();
+      Glom::Utils::dialog_run_with_help(dialog, "dialog_relationships_overview");
 
       remove_view(dialog);
       delete dialog;
@@ -1131,7 +1306,7 @@ void Frame_Glom::on_menu_developer_users()
   add_view(dialog); //Give it access to the document.
   dialog->load_from_document(); //Update the UI now that it has the document.
 
-  dialog->run();
+  Glom::Utils::dialog_run_with_help(dialog, "window_groups");
   remove_view(dialog);
   delete dialog;
 
@@ -1173,6 +1348,20 @@ void Frame_Glom::on_menu_developer_reports()
   }
 }
 
+void Frame_Glom::on_menu_developer_script_library()
+{
+  Dialog_ScriptLibrary* dialog = 0;
+  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_script_library");
+  refXml->get_widget_derived("dialog_script_library", dialog);
+  dialog->set_transient_for(*(get_app_window()));
+  add_view(dialog); //Give it access to the document.
+  dialog->load_from_document();
+  Glom::Utils::dialog_run_with_help(dialog, "dialog_script_library"); //TODO: Create the help section.
+  dialog->save_to_document();
+  remove_view(dialog);
+  delete dialog;
+}
+
 
 void Frame_Glom::on_box_reports_selected(const Glib::ustring& report_name)
 {
@@ -1191,6 +1380,10 @@ void Frame_Glom::on_developer_dialog_hide()
 {
   //The dababase structure might have changed, so refresh the data view:
   show_table(m_table_name);
+
+  //TODO: This is a bit of a hack. It's not always useful to do this:
+  if(m_dialog_addrelatedtable)
+    m_dialog_addrelatedtable->set_fields(m_table_name);
 }
 
 bool Frame_Glom::connection_request_password_and_choose_new_database_name()
@@ -1198,7 +1391,7 @@ bool Frame_Glom::connection_request_password_and_choose_new_database_name()
   //Ask for connection details:
   m_pDialogConnection->load_from_document(); //Get good defaults.
   m_pDialogConnection->set_transient_for(*get_app_window());
-  int response = m_pDialogConnection->run();
+  int response = Glom::Utils::dialog_run_with_help(m_pDialogConnection, "dialog_connection");
   m_pDialogConnection->hide();
 
   if(response == Gtk::RESPONSE_OK)
@@ -1237,14 +1430,14 @@ bool Frame_Glom::connection_request_password_and_choose_new_database_name()
         {
           //Warn the user, and let him try again:
           m_pDialogConnectionFailed->set_transient_for(*get_app_window());
-          int response = m_pDialogConnectionFailed->run();
+          int response = Glom::Utils::dialog_run_with_help(m_pDialogConnectionFailed, "dialog_error_connection");
           m_pDialogConnectionFailed->hide();
 
           //TODO: Combine these into one dialog.
           if(response != Gtk::RESPONSE_OK)
             return false; //The user cancelled.
 
-          response = m_pDialogConnection->run();
+          response = Glom::Utils::dialog_run_with_help(m_pDialogConnection, "dialog_connection");
           m_pDialogConnection->hide();
           if(response != Gtk::RESPONSE_OK)
             return false; //The user cancelled.
@@ -1282,7 +1475,7 @@ bool Frame_Glom::connection_request_password_and_attempt()
     //Ask for connection details:
     m_pDialogConnection->load_from_document(); //Get good defaults.
     m_pDialogConnection->set_transient_for(*get_app_window());
-    int response = m_pDialogConnection->run();
+    int response = Glom::Utils::dialog_run_with_help(m_pDialogConnection, "dialog_connection");
     m_pDialogConnection->hide();
 
     if(response == Gtk::RESPONSE_OK)
@@ -1301,7 +1494,7 @@ bool Frame_Glom::connection_request_password_and_attempt()
         {
           //Warn the user, and let him try again:
           m_pDialogConnectionFailed->set_transient_for(*get_app_window());
-          int response = m_pDialogConnectionFailed->run();
+          int response = Glom::Utils::dialog_run_with_help(m_pDialogConnectionFailed, "dialog_error_connection");
           m_pDialogConnectionFailed->hide();
 
           if(response != Gtk::RESPONSE_OK)
@@ -1379,12 +1572,19 @@ bool Frame_Glom::create_database(const Glib::ustring& database_name, const Glib:
 
       Bakery::BusyCursor busycursor(*pWindowApp);
 
+      //std::cout << "Frame_Glom::create_database():  debug: before calling sharedconnection->get_gda_connection." << std::endl;
+
       Glib::RefPtr<Gnome::Gda::Connection> connection = sharedconnection->get_gda_connection();
       if(connection)
       {
         const bool test = connection->create_database(database_name);
         if(!test)
         {
+          //TODO: Discover the cause of the error somehow.
+          //I think a failure here might be caused by installing unstable libgda, which seems to affect stable libgda-1.2.
+          //Doing a "make install" in libgda-1.2 seems to fix this:
+          std::cerr << "Frame_Glom::create_database():  Gnome::Gda::Connection::create_database(" << database_name << ") failed." << std::endl;
+
           //Tell the user:
           Gtk::Dialog* dialog = 0;
           try
@@ -1392,7 +1592,7 @@ bool Frame_Glom::create_database(const Glib::ustring& database_name, const Glib:
             Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_error_create_database");
             refXml->get_widget("dialog_error_create_database", dialog);
             dialog->set_transient_for(*pWindowApp);
-            dialog->run();
+            Glom::Utils::dialog_run_with_help(dialog, "dialog_error_create_database");
             delete dialog;
           }
           catch(const Gnome::Glade::XmlError& ex)
@@ -1411,9 +1611,6 @@ bool Frame_Glom::create_database(const Glib::ustring& database_name, const Glib:
     }
   }
 
-  std::cout << "Frame_Glom::create_database(): debug 5" << std::endl;
-
-
   //Connect to the actual database:
   ConnectionPool* connection_pool = ConnectionPool::get_instance();
   connection_pool->set_database(database_name);
@@ -1468,7 +1665,7 @@ bool Frame_Glom::create_database(const Glib::ustring& database_name, const Glib:
 
 void Frame_Glom::on_menu_report_selected(const Glib::ustring& report_name)
 {
-  const Privileges table_privs = get_current_privs(m_table_name);
+  const Privileges table_privs = Privs::get_current_privs(m_table_name);
 
   //Don't try to print tables that the user can't view.
   if(!table_privs.m_view)
@@ -1477,12 +1674,16 @@ void Frame_Glom::on_menu_report_selected(const Glib::ustring& report_name)
     return;
   }
 
-  sharedptr<Report> report = get_document()->get_report(m_table_name, report_name);
+  Document_Glom* document = get_document();
+  sharedptr<Report> report = document->get_report(m_table_name, report_name);
   if(!report)
     return;
 
   FoundSet found_set = m_Notebook_Data.get_found_set();
-  report_build(found_set, report, get_app_window()); //TODO: Use found set's where_clause.
+
+  ReportBuilder report_builder;
+  report_builder.set_document(document);
+  report_builder.report_build(found_set, report, get_app_window()); //TODO: Use found set's where_clause.
 }
 
 void Frame_Glom::on_dialog_layout_report_hide()
@@ -1590,3 +1791,5 @@ void Frame_Glom::on_button_find_all()
   show_table(m_table_name);
 }
 
+} //namespace Glom
+
