@@ -84,6 +84,17 @@ public:
   bool get_is_example_file() const;
   void set_is_example_file(bool value = true);
 
+  /* Get version of the document format used for this document.
+   *  This can increase when the file has been re-saved.
+   *  See get_latest_known_document_format_version().
+   *  Old versions of the application cannot open documents with a newer document format,
+   *  so saving with a version of the application that has a newer document format will 
+   *  make it impossible to open the document in a version of the application with an older document format.
+   */  
+  guint get_document_format_version();
+
+  static guint get_latest_known_document_format_version();
+
   virtual void set_connection_server(const Glib::ustring& strVal);
   virtual void set_connection_user(const Glib::ustring& strVal);
   virtual void set_connection_database(const Glib::ustring& strVal);
@@ -198,11 +209,16 @@ public:
 
   virtual Glib::ustring get_name() const; //override.
 
-  virtual Glib::ustring get_default_table() const;
-  virtual Glib::ustring get_first_table() const;
+  Glib::ustring get_default_table() const;
+  Glib::ustring get_first_table() const;
 
-  virtual Glib::ustring get_database_title() const;
-  virtual void set_database_title(const Glib::ustring& title);
+  Glib::ustring get_database_title() const;
+  void set_database_title(const Glib::ustring& title);
+
+  std::vector<Glib::ustring> get_library_module_names() const;
+  void set_library_module(const Glib::ustring& name, const Glib::ustring& script);
+  Glib::ustring get_library_module(const Glib::ustring& name) const;
+  void remove_library_module(const Glib::ustring& name);
 
   /// These are only used when recreating a database from an example file. The actualy access-control is on the server, of course.
   typedef std::list<GroupInfo> type_list_groups;
@@ -304,6 +320,9 @@ protected:
   static void set_node_attribute_value_as_value(xmlpp::Element* node, const Glib::ustring& strAttributeName, const Gnome::Gda::Value& value, Field::glom_field_type field_type);
   static Gnome::Gda::Value get_node_attribute_value_as_value(const xmlpp::Element* node, const Glib::ustring& strAttributeName, Field::glom_field_type field_type);
 
+  Glib::ustring get_child_text_node(const xmlpp::Element* node, const Glib::ustring& child_node_name) const;
+  void set_child_text_node(xmlpp::Element* node, const Glib::ustring& child_node_name, const Glib::ustring& text);
+
   AppState m_app_state;
   type_signal_userlevel_changed m_signal_userlevel_changed;
 
@@ -359,10 +378,14 @@ protected:
   Glib::ustring m_database_title;
   Glib::ustring m_translation_original_locale;
 
+  typedef std::map<Glib::ustring, Glib::ustring> type_map_library_scripts;
+  type_map_library_scripts m_map_library_scripts;
+
   bool m_block_cache_update; //For efficiency.
   bool m_block_modified_set;
   bool m_allow_auto_save;
   bool m_is_example;
+  guint m_document_format_version;
 
   Gtk::Window* m_parent_window; //Needed by BusyCursor.(gtype_0)
 };
