@@ -20,12 +20,17 @@
 
 #include "box_data_list.h"
 #include <glom/libglom/data_structure/glomconversions.h>
+#include <glom/reports/report_builder.h>
 #include "dialog_layout_list.h"
+#include <glom/glom_privs.h>
 #include <bakery/App/App_Gtk.h> //For util_bold_message().
 //#include <../utility_widgets/db_adddel/glom_db_treemodel.h> //For DbTreeModel.
 #include <sstream> //For stringstream
 #include <glibmm/i18n.h>
 
+namespace Glom
+{
+
 Box_Data_List::Box_Data_List()
 : m_has_one_or_more_records(false),
   m_read_only(false)
@@ -76,7 +81,7 @@ Box_Data_List::~Box_Data_List()
 
 void Box_Data_List::enable_buttons()
 {
-  const Privileges table_privs = get_current_privs(m_table_name);
+  const Privileges table_privs = Privs::get_current_privs(m_table_name);
 
     //Enable/Disable record creation and deletion:
   bool allow_create = !m_read_only;
@@ -99,6 +104,7 @@ void Box_Data_List::refresh_data_from_database_blank()
   found_set.m_where_clause = Glib::ustring();
   m_AddDel.set_found_set(found_set);
 
+  std::cout << "debug: Box_Data_List::refresh_data_from_database_blank(): before refresh_from_database_blank()." << std::endl;
   m_AddDel.refresh_from_database_blank();
   m_found_set = found_set;
 }
@@ -115,8 +121,6 @@ bool Box_Data_List::fill_from_database()
 
     Box_Data::fill_from_database();
 
-    m_AddDel.remove_all();
-
     //Field Names:
     //create_layout();
 
@@ -125,7 +129,7 @@ bool Box_Data_List::fill_from_database()
       //Glib::RefPtr<Gnome::Gda::Connection> connection = sharedconnection->get_gda_connection();
 
     //Do not try to show the data if the user may not view it:
-    const Privileges table_privs = get_current_privs(m_table_name);
+    const Privileges table_privs = Privs::get_current_privs(m_table_name);
 
     enable_buttons();
 
@@ -211,7 +215,7 @@ void Box_Data_List::on_adddel_user_requested_add()
 
 void Box_Data_List::on_adddel_user_requested_edit(const Gtk::TreeModel::iterator& row)
 {
-  Gnome::Gda::Value primary_key_value = m_AddDel.get_value_key(row); //The primary key is in the key.
+  const Gnome::Gda::Value primary_key_value = m_AddDel.get_value_key(row); //The primary key is in the key.
 
   signal_user_requested_details().emit(primary_key_value);
 }
@@ -244,7 +248,7 @@ void Box_Data_List::set_primary_key_value(const Gtk::TreeModel::iterator& row, c
   m_AddDel.set_value_key(row, value);
 }
 
-void Box_Data_List::on_adddel_user_added(const Gtk::TreeModel::iterator& row, guint col_with_first_value)
+void Box_Data_List::on_adddel_user_added(const Gtk::TreeModel::iterator& row, guint /* col_with_first_value */)
 {
   //std::cout << "Box_Data_List::on_adddel_user_added" << std::endl;
 
@@ -268,7 +272,7 @@ void Box_Data_List::on_adddel_user_added(const Gtk::TreeModel::iterator& row, gu
   }
 
   //If no primary key value is available yet, then don't add the record yet:
-  if(!GlomConversions::value_is_empty(primary_key_value))
+  if(!Conversions::value_is_empty(primary_key_value))
   {
     sharedptr<SharedConnection> sharedconnection = connect_to_server(get_app_window()); //Keep it alive while we need the data_model.
     if(sharedconnection)
@@ -278,7 +282,7 @@ void Box_Data_List::on_adddel_user_added(const Gtk::TreeModel::iterator& row, gu
       if(!check_entered_value_for_uniqueness(m_table_name, layout_field, primary_key_value, get_app_window()))
       {
         //Revert to a blank value.
-        primary_key_value = GlomConversions::get_empty_value(layout_field->get_full_field_details()->get_glom_type());
+        primary_key_value = Conversions::get_empty_value(layout_field->get_full_field_details()->get_glom_type());
         set_entered_field_data(row, layout_field, primary_key_value);
         return;
       }
@@ -346,7 +350,7 @@ void Box_Data_List::on_adddel_user_changed(const Gtk::TreeModel::iterator& row,
   const Gnome::Gda::Value parent_primary_key_value = get_primary_key_value(row);
   sharedptr<const LayoutItem_Field> layout_field = m_AddDel.get_column_field(col);
 
-  if(!GlomConversions::value_is_empty(parent_primary_key_value)) //If the record's primary key is filled in:
+  if(!Conversions::value_is_empty(parent_primary_key_value)) //If the record's primary key is filled in:
   {
     //Just update the record:
     try
@@ -446,7 +450,7 @@ void Box_Data_List::on_adddel_user_changed(const Gtk::TreeModel::iterator& row,
     on_adddel_user_added(row, col);
     
     const Gnome::Gda::Value primaryKeyValue = get_primary_key_value(row); //TODO_Value
-    if(!(GlomConversions::value_is_empty(primaryKeyValue))) //If the Add succeeeded:
+    if(!(Conversions::value_is_empty(primaryKeyValue))) //If the Add succeeeded:
     {
       if(!(layout_field->get_full_field_details()->get_primary_key())) //Don't try to re-set the primary key field, because we just inserted the record with it.
       {
@@ -569,7 +573,7 @@ Gnome::Gda::Value Box_Data_List::get_primary_key_value_first()
     while(iter != model->children().end())
     {
       Gnome::Gda::Value value = get_primary_key_value(iter);
-      if(GlomConversions::value_is_empty(value))
+      if(Conversions::value_is_empty(value))
       {
        //std::cout << "Box_Data_List(): get_primary_key_value_first() iter val is NULL" << std::endl;
         ++iter;
@@ -606,6 +610,53 @@ bool Box_Data_List::get_showing_multiple_records() const
   return m_AddDel.get_count() > 1;
 }
 
+void Box_Data_List::create_layout_add_group(const sharedptr<LayoutGroup>& layout_group)
+{
+  if(!layout_group)
+    return;
+
+  LayoutGroup::type_map_items child_items = layout_group->get_items();
+  for(LayoutGroup::type_map_items::const_iterator iter = child_items.begin(); iter != child_items.end(); ++iter)
+  {
+    sharedptr<LayoutItem> child_item = iter->second;
+
+    sharedptr<LayoutGroup> child_group = sharedptr<LayoutGroup>::cast_dynamic(child_item);
+    if(child_group)
+    {
+      //std::cout << "debug: Start Adding child group." << std::endl;
+      create_layout_add_group(child_group);
+      //std::cout << "debug: End Adding child group." << std::endl;
+    }
+    else
+    {
+      if(m_read_only)
+        child_item->set_editable(false);
+
+      //std::cout << "debug: adding column: " << child_item->get_name() << std::endl;
+
+      sharedptr<LayoutItem_Field> child_field = sharedptr<LayoutItem_Field>::cast_dynamic(child_item);
+      if(child_field)
+      {
+        //Check that the field really exists, to avoid SQL errors.
+        //This could probably only happen if we have failed to rename something everywhere, when the user has renamed something.
+        if(!get_field_exists_in_database(child_field->get_table_used(m_table_name), child_field->get_name()))
+        {
+          std::cerr << "debug: Box_Data_List::create_layout_add_group(): Field does not exist in database: table_name=" << child_field->get_table_used(m_table_name) << ", field_name=" << child_field->get_name() << std::endl;
+          continue;
+        }
+      }
+
+      m_AddDel.add_column(child_item);
+    }
+  }
+}
+
+Document_Glom::type_mapLayoutGroupSequence Box_Data_List::create_layout_get_layout()
+{
+  //Overriden in Box_Data_List_Related:
+  return get_data_layout_groups(m_layout_name); 
+}
+
 void Box_Data_List::create_layout()
 {
   Box_Data::create_layout(); //Fills m_TableFields.
@@ -619,6 +670,7 @@ void Box_Data_List::create_layout()
 
     m_AddDel.set_table_name(m_table_name);
 
+
     sharedptr<Field> field_primary_key = get_field_primary_key_for_table(m_table_name);
     if(!field_primary_key)
     {
@@ -627,32 +679,38 @@ void Box_Data_List::create_layout()
     else
     {
       m_AddDel.set_key_field(field_primary_key);
+ 
+      //This map of layout groups will also contain the field information from the database:
+      Document_Glom::type_mapLayoutGroupSequence layout_groups = create_layout_get_layout();
 
-      m_FieldsShown = get_fields_to_show();
-
-      //Add extra possibly-non-visible columns that we need:
-      //TODO: Only add it if it is not already there.
-      sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
-      layout_item->set_hidden();
-      layout_item->set_full_field_details(m_AddDel.get_key_field());
-      m_FieldsShown.push_back(layout_item);
-
-
-      //Add a column for each table field:
-      for(type_vecLayoutFields::iterator iter = m_FieldsShown.begin(); iter != m_FieldsShown.end(); ++iter)
+      //int debug_count = 0;
+      for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
       {
-        sharedptr<LayoutItem_Field> field = *iter;
-        if(m_read_only)
-          field->set_editable(false);
+        //std::cout << "Box_Data_List::create_layout() group number=" << debug_count;
+        //debug_count++;
+        //iter->second->debug();
 
-        //std::cout << "Adding field: name=" << field->get_name() << ", titleorname=" << field->get_title_or_name() << std::endl;
-        m_AddDel.add_column(field);
+        create_layout_add_group(iter->second);
       }
+    }
 
-      m_AddDel.set_found_set(m_found_set);
 
-      m_AddDel.set_columns_ready();
-     }
+    m_FieldsShown = get_fields_to_show();
+
+    //Add extra possibly-non-visible columns that we need:
+    //TODO: Only add it if it is not already there.
+    sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
+    layout_item->set_hidden();
+    layout_item->set_full_field_details(m_AddDel.get_key_field());
+    m_FieldsShown.push_back(layout_item);
+
+    m_AddDel.add_column(layout_item);
+
+    m_AddDel.set_found_set(m_found_set);
+
+    //Column-creation happens in fill_database() instead:
+    //otherwise the treeview will be filled twice.
+    //m_AddDel.set_columns_ready();
   }
 
 }
@@ -685,7 +743,7 @@ bool Box_Data_List::get_field_primary_key_index(guint& field_column) const
 
 void Box_Data_List::print_layout()
 {
-  const Privileges table_privs = get_current_privs(m_table_name);
+  const Privileges table_privs = Privs::get_current_privs(m_table_name);
 
   //Don't try to print tables that the user can't view.
   if(!table_privs.m_view)
@@ -705,7 +763,9 @@ void Box_Data_List::print_layout()
       report_temp->m_layout_group->add_item(*iter);
     }
 
-    report_build(m_found_set, report_temp, get_app_window());
+    ReportBuilder report_builder;
+    report_builder.set_document(get_document());
+    report_builder.report_build(m_found_set, report_temp, get_app_window());
   }
 }
 
@@ -748,6 +808,7 @@ void Box_Data_List::get_record_counts(gulong& total, gulong& found) const
     refModelDerived->get_record_counts(total, found);
 }
 
+} //namespace Glom
 
 
 
