@@ -23,16 +23,19 @@
 
 #include <gtkmm.h>
 #include "../../utility_widgets/combo_textglade.h"
-#include "combo_lookup_relationship.h"
+#include "../../combobox_relationship.h"
 #include "combo_fieldtype.h"
 #include "../../utility_widgets/table_columns.h"
 //#include "../../utility_widgets/entry_numerical.h"
 #include "../../utility_widgets/dialog_properties.h"
 #include "../../utility_widgets/datawidget.h"
-#include "../../data_structure/field.h"
+#include <glom/libglom/data_structure/field.h>
 #include "../../layout_item_dialogs/box_formatting.h"
 #include "../../base_db.h"
+#include <gtksourceviewmm/sourceview.h>
 
+namespace Glom
+{
 
 class Dialog_FieldDefinition
  : public Dialog_Properties,
@@ -42,8 +45,8 @@ public:
   Dialog_FieldDefinition(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
   virtual ~Dialog_FieldDefinition();
 
-  virtual void set_field(const Field& field, const Glib::ustring& table_name);
-  virtual Field get_field() const;
+  virtual void set_field(const sharedptr<const Field>& field, const Glib::ustring& table_name);
+  virtual sharedptr<Field> get_field() const; //TODO_FieldShared
 
 protected:
 
@@ -51,7 +54,8 @@ protected:
   void on_combo_type_changed();
   void on_combo_lookup_relationship_changed();
   void on_check_lookup_toggled();
-  void on_check_calculate_toggled();
+  void on_radio_calculate_toggled();
+  void on_radio_userentry_toggled();
   void on_button_edit_calculation();
 
   //void on_foreach(Gtk::Widget& widget);
@@ -67,15 +71,18 @@ protected:
   Gtk::CheckButton* m_pCheck_PrimaryKey;
   Gtk::CheckButton* m_pCheck_AutoIncrement;
 
-  Gtk::VBox* m_pBox_DefaultValue;
+  Gtk::VBox* m_pBox_ValueTab;
+
+  Gtk::RadioButton* m_pRadio_UserEntry;
+  Gtk::Alignment* m_pAlignment_UserEntry;
   Gtk::CheckButton* m_pCheck_Lookup;
   Gtk::Table* m_pTable_Lookup;
-  Combo_LookupRelationship* m_pCombo_LookupRelationship;
+  ComboBox_Relationship* m_pCombo_LookupRelationship;
   Combo_TextGlade* m_pCombo_LookupField;
 
-  Gtk::CheckButton* m_pCheck_Calculate;
+  Gtk::RadioButton* m_pRadio_Calculate;
   Gtk::Alignment* m_pAlignment_Calculate;
-  Gtk::TextView* m_pTextView_Calculation;
+  gtksourceview::SourceView* m_pTextView_Calculation;
   Gtk::Button* m_pButton_EditCalculation;
 
   Gtk::Entry* m_pEntry_Title;
@@ -85,8 +92,10 @@ protected:
   Gtk::VBox* m_box_formatting_placeholder;
   Box_Formatting* m_box_formatting;
 
-  Field m_Field;
+  sharedptr<Field> m_Field;
   Glib::ustring m_table_name;
 };
 
+} //namespace Glom
+
 #endif //DIALOG_FIELDDEFINITION_H
